#if !defined(__PPMD_CONSTANTS_H__)
#define __PPMD_CONSTANTS_H__


enum { UNIT_SIZE=12, N1=4, N2=4, N3=4, N4=26, N_INDEXES=38 };

enum { UP_FREQ=5, INT_BITS=7, PERIOD_BITS=7, TOT_BITS=14,
	INTERVAL=128, BIN_SCALE=16384, MAX_FREQ=124, O_BOUND=9 };


#endif
